/**
 * Copyright (C) 2015 Deepin Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 **/

#include <QPixmap>
#include <QDebug>
#include <QTimer>

#include <dtextbutton.h>
#include <dseparatorhorizontal.h>
#include <dcolorpicker.h>
#include <dsegmentedcontrol.h>
#include <dbuttonlist.h>
#include <dbuttongrid.h>
#include <dlabel.h>
#include <dswitchbutton.h>

#include "moduleheader.h"
#include "constants.h"

#include "grubwidget.h"
#include "dbustheme.h"
#include "bootmenulist.h"

GrubWidget::GrubWidget(QWidget *parent):
    ScrollFrame(parent),
    m_header(new ModuleHeader(tr("Boot Menu"))),
    m_arrowDefaultBoot(new DArrowLineExpand()),
    m_arrowBootDelay(new DArrowLineExpand()),
    m_arrowTextColor(new DArrowLineExpand()),
    m_arrowSelectedTextColor(new DArrowLineExpand()),
    m_themeDbus(new GrubThemeDbus(this)),
    m_grubDbus(new GrubDbus(this)),
    m_grubBackground(new GrubBackground(m_themeDbus)),
    m_bootEntryList(new DButtonList())
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    init();
    m_grubBackground->installEventFilter(this);

    connect(m_themeDbus, &GrubThemeDbus::updatingChanged, this, &GrubWidget::updatingChanged);
    connect(m_grubDbus, &GrubDbus::UpdatingChanged, this, &GrubWidget::updatingChanged);
}

GrubWidget::~GrubWidget()
{

}

bool GrubWidget::eventFilter(QObject *obj, QEvent *e)
{
    if (obj == m_grubBackground) {
        if (e->type() == QEvent::Resize) {
            m_tooltip->move(0, m_grubBackground->height() - m_tooltip->height());
        }
    }

    return false;
}

void GrubWidget::init()
{
    m_header->setFixedSize(DCC::ModuleContentWidth, DTK_WIDGET_NAMESPACE::CONTENT_HEADER_HEIGHT);

    connect(m_header, &ModuleHeader::resetButtonClicked, m_grubDbus, &GrubDbus::Reset);

    m_tooltip = new QLabel(m_grubBackground);
    m_tooltip->setStyleSheet("QLabel{background:rbga(0,0,0,110);color:white;}");
    m_tooltip->resize(DCC::ModuleContentWidth, 20);
    m_tooltip->setText(tr("Drag and drop an image to change background."));
    m_tooltip->setAlignment(Qt::AlignCenter);

    DSwitchButton *m_enableThemeSwitch = new DSwitchButton;
    DHeaderLine *m_enableThemeLine = new DHeaderLine;
    m_enableThemeLine->setTitle(tr("Theme"));
    m_enableThemeLine->setContent(m_enableThemeSwitch);
    m_enableThemeSwitch->setChecked(true);
    m_enableThemeLine->setFixedHeight(RADIO_ITEM_HEIGHT);

    connect(m_enableThemeSwitch, &DSwitchButton::checkedChanged,
            m_grubDbus, &GrubDbus::setEnableTheme);

    connect(m_grubDbus, &GrubDbus::EnableThemeChanged,
            m_enableThemeSwitch, &DSwitchButton::setChecked);

    QStringList title_list = m_grubDbus->GetSimpleEntryTitles();
    m_bootEntryList->setItemWidth(DCC::ModuleContentWidth);
    m_bootEntryList->addButtons(title_list);
    m_bootEntryList->setFixedSize(DCC::ModuleContentWidth, m_bootEntryList->count()*MENU_ITEM_HEIGHT);
    m_bootMenuTitle = new BootMenuList(m_grubBackground);
    m_bootMenuTitle->addButtons(title_list);
    m_bootMenuTitle->move(50, 50);
    m_bootMenuTitle->setNormalColor(m_themeDbus->itemColor());
    m_bootMenuTitle->setEnabledColor(m_themeDbus->selectedItemColor());
    connect(m_themeDbus, &GrubThemeDbus::itemColorChanged, m_bootMenuTitle, &BootMenuList::setNormalColor);
    connect(m_themeDbus, &GrubThemeDbus::selectedItemColorChanged, m_bootMenuTitle, &BootMenuList::setEnabledColor);

    setDefaultEntry(m_grubDbus->defaultEntry());
    connect(m_grubDbus, &GrubDbus::DefaultEntryChanged, this, &GrubWidget::setDefaultEntry);
    connect(m_bootEntryList, &DButtonList::buttonChecked, m_grubDbus, &GrubDbus::setDefaultEntry);

    m_arrowDefaultBoot->setTitle(tr("Default Boot"));
    m_arrowDefaultBoot->setContent(m_bootEntryList);

    DButtonGrid *timeout_select = new DButtonGrid(1, 7);
    m_timeoutList << "1s" << "5s" << "10s" << "15s" << "20s" << "25s" << "30s";
    timeout_select->addButtons(m_timeoutList);
    timeout_select->setItemSize(30, RADIO_ITEM_HEIGHT);
    timeout_select->setFixedHeight(RADIO_ITEM_HEIGHT);
    timeout_select->checkButtonByIndex(m_timeoutList.indexOf(QString("%1s").arg(m_grubDbus->timeout())));
    connect(m_grubDbus, &GrubDbus::TimeoutChanged, [ = ](int timeout) {
        timeout_select->checkButtonByIndex(m_timeoutList.indexOf(QString("%1s").arg(timeout)));
    });
    connect(timeout_select, &DButtonGrid::buttonChecked, [ = ](QString title) {
        m_grubDbus->setTimeout(title.replace('s', "").toInt());
    });

    m_arrowBootDelay->setTitle(tr("Boot delay"));
    m_arrowBootDelay->setContent(timeout_select);

    DColorPicker *picker1 = new DColorPicker(10, this);
    picker1->setCurrentColor(m_themeDbus->itemColor());
    connect(picker1, &DColorPicker::currentColorChanged, m_themeDbus, &GrubThemeDbus::setItemColor);
    connect(m_themeDbus, &GrubThemeDbus::itemColorChanged, picker1, &DColorPicker::setCurrentColor);

    m_arrowTextColor->setTitle(tr("Text Color"));
    m_arrowTextColor->setContent(picker1);

    DColorPicker *picker2 = new DColorPicker(10, this);
    picker2->setCurrentColor(m_themeDbus->selectedItemColor());
    connect(picker2, &DColorPicker::currentColorChanged, m_themeDbus, &GrubThemeDbus::setSelectedItemColor);
    connect(m_themeDbus, &GrubThemeDbus::selectedItemColorChanged, picker2, &DColorPicker::setCurrentColor);

    m_arrowSelectedTextColor->setTitle(tr("Selected Text Color"));
    m_arrowSelectedTextColor->setContent(picker2);

    DLabel *hits = new DLabel;
    hits->setText(tr("The startup speed can be improved by disabling the theme"));
    hits->setContentsMargins(16, 10, 16, 10);
    hits->setWordWrap(true);
    hits->setStyleSheet("font-size:11px;color:grey;");

    headerLayout()->addWidget(m_header);

    mainLayout()->setSpacing(0);
    mainLayout()->addWidget(new DSeparatorHorizontal());
    mainLayout()->addWidget(m_grubBackground);
    mainLayout()->addWidget(new DSeparatorHorizontal());
    mainLayout()->addWidget(m_enableThemeLine);
    mainLayout()->addWidget(new DSeparatorHorizontal());
    mainLayout()->addWidget(m_arrowDefaultBoot);
    mainLayout()->addWidget(m_arrowBootDelay);
    mainLayout()->addWidget(m_arrowTextColor);
    mainLayout()->addWidget(m_arrowSelectedTextColor);
    mainLayout()->addWidget(hits);
}

void GrubWidget::setDefaultEntry(const QString &entry)
{
    QStringList title_list = m_grubDbus->GetSimpleEntryTitles();
    int default_index = title_list.indexOf(entry);

    if (default_index >= 0 && default_index < m_bootEntryList->count()) {
        m_bootEntryList->checkButtonByIndex(default_index);
        m_bootMenuTitle->checkButtonByIndex(default_index);
    }
}

void GrubWidget::updatingChanged()
{
    m_tooltip->setStyleSheet(m_tooltip->styleSheet() + "QLabel{color:#DF8000;}");

    if (m_themeDbus->updating() || m_grubDbus->updating()) {
        m_tooltip->setText(tr("Updating..."));
    } else {
        m_tooltip->setText(tr("Successfully updated, reboot to view."));
        QTimer::singleShot(2000, this, SLOT(resetTooltip()));
    }
}

void GrubWidget::resetTooltip()
{
    m_tooltip->setStyleSheet(m_tooltip->styleSheet() + "QLabel{color:white;}");
    m_tooltip->setText(tr("Drag and drop an image to change background."));
}
